var ZeedhiDirectives = angular.module("ZeedhiDirectives");

ZeedhiDirectives.directive('zhSchemaValidator', function(templateManager, DevToolsService, ElementToMetadataService, ApplicationContext) {
	return {
		link: function link($scope, element, attrs) {

			function getWindowController(){
				return ApplicationContext.WindowController;
			}

			function getAndFormatType(path){
				var type = "container";
				var pathLength = path && path.length;
				if(pathLength >= 2){
					type = path[pathLength - 2];
					var lastChar = type.length - 1;
					if(type[lastChar] == "s"){
						type = type.substring(0, lastChar);
					}
				}
				return type;
			}

			function getElementName(jsonElem, type){
				return jsonElem.label || jsonElem.name || jsonElem.id || type.toUpperCase();
			}

			function getFormatedDataPaht(dataPath) {
				dataPath = dataPath || "";
				if(dataPath){
					dataPath = dataPath.replace(/^\//, '');
					dataPath = "In "+dataPath+" - ";
				}
				return dataPath;
			}

			function formatErrorsMessage(errors, path, jsonElement){
				var type = getAndFormatType(path);
				var message = "In " + type;
				message += " " + getElementName(jsonElement, type);
				message += "(" + path.join("/") + "): ";
				errors.forEach(function(error){
					var dataPath = getFormatedDataPaht(error.dataPath);
					message += "\n\t"+dataPath+error.message;
				});
				return message;
			}

			function showErrorsInTree(path, message){
				var type = getAndFormatType(path);
				var object = $scope[type];
				if(object){
					var invalidItem = {
						path: path,
						errorMessage: message
					};
					getWindowController().addInvalidItem(invalidItem);
				}
			}

			function showErrors(errors, path, jsonElement){
				var message = formatErrorsMessage(errors, path, jsonElement);
				console.warn(message);
				showErrorsInTree(path, message);
			}

			function clearDependencieErrors(errors, schema){
				errors = errors.filter(function(error){
					var propertieKeys = "";
					if(schema.properties[error.params.missing]){
						propertieKeys = Object.keys(schema.properties[error.params.missing])	;
					}
					return (error.code != 304 || propertieKeys.indexOf('default') < 0);
				});
				return errors;
			}

			var validateIfInDevMode = UtilDevTools.buildInDeveloperModeMethod(function(){
				var schemaFileName = attrs.zhSchemaValidator;
				var schemaPath = templateManager.getDocsUrl(schemaFileName);
				var metadataElement = ElementToMetadataService.getElementToMetadata($scope, templateManager.container);
				var jsonElement = metadataElement.jsonElement;

				if(jsonElement){
					DevToolsService.requestMetadataDeveloper(schemaPath).then(function(schema) {
						var errors = tv4.validateMultiple(jsonElement, schema.data).errors || [];
						errors = clearDependencieErrors(errors, schema.data);
						if(errors.length){
							showErrors(errors, metadataElement.path, jsonElement);
						}
					});
				}
			});
			UtilDevTools.buildMainWindowMethod(validateIfInDevMode)();
		}
	};
});